%start CommandSequence
%%

CommandSequence -> Result <LinkedList<Result<LineOfCode, CompilerError>>, CompilerError>:
	CommandLine CommandSequence
		{
			let mut list = $2?;
			list.push_front($1);
			Ok(list)
		}
	| CommandLine
		{
			let mut list : LinkedList<Result<LineOfCode, CompilerError>> = LinkedList::new();
			list.push_front($1);
			Ok(list)
		}
	;

CommandLine -> Result<LineOfCode, CompilerError>:
	Command 'NEWLINE'
		{
			Ok(LineOfCode{line : 42, code: $1})
		}
	| 'NEWLINE'
		{
			Err(CompilerError {pos: $1.unwrap().span(), description: CompilerErrorType::MiscError})
		}
	;

/*Comment -> Result<(), ()>:
	';' 'text':
		{
			Ok(())
		}
	;*/

Command -> Result <Code, CompilerError>:
	'SetPtr' PointerId Symbol
		{
			Ok(Code::Instruction(InstructionType::SetPtr{which: $2?, address: $3?}))
		}
	;

PointerId -> Result<u16, CompilerError>:
	Literal
		{
			let num = $1?;
			if num > 0 && num <= 1 {
				Ok(num)
			} else {
				//Err(CompilerError {pos: $lexer.line_col($1.unwrap().span()), description: CompilerErrorType::MiscError})
				Ok(42) // TODO: This should raise an error, but I don't know the span anymore -.-
			}
		}
	;

Symbol -> Result<Symbol, CompilerError>:
	Literal
		{
			Ok(Symbol::ResolvedSymbol{address: $1?})
		}
	| Reference
		{
			Ok(Symbol::UnresolvedSymbol{symbol: $1?})
		}
	;

Reference -> Result<String, CompilerError>:
	'Reference'
		{
			Ok($lexer.span_str($1.unwrap().span())[1..].to_string())
		}
	;

Literal -> Result<u16, CompilerError>:
	'NUM-HEX'
		{
			parse_number(16, $1.unwrap().span(), &$lexer.span_str($1.unwrap().span())[2..]) // TODO: Change error to include line/col etc.
		}
	| 'NUM-DEC'
		{
			parse_number(10, $1.unwrap().span(), &$lexer.span_str($1.unwrap().span()))
		}
	;

%%

use crate::compiler_types::*;
use std::collections::LinkedList;

fn parse_number(radix: u32, span: lrpar::Span, s: &str) -> Result<u16, CompilerError> {
	//Ok(u16::from_str_radix(s, radix)?)
	//u16::from_str_radix(s, radix).map_err(CompilerError{description: CompilerErrorType::MiscError})
	u16::from_str_radix(s, radix).map_err(|e| CompilerError{pos: span, description: e.into()})
}
