use std::num;

#[derive(Debug)]
pub enum Symbol {
	UnresolvedSymbol{symbol: String},
	ResolvedSymbol{address: u16}
}

#[derive(Debug)]
pub enum JumpCondition {
	Always,
	OnMatch,
	OnTimer
}

#[derive(Debug)]
pub enum InstructionType {
	SetPtr{which: u16, address: Symbol},
	Output{length: Symbol},
	Match{length: Symbol},
	Write{length: Symbol},
	Timer{time: Symbol},
	Page{page: Symbol},
	Return,
	Input{length: Symbol},
	Jump{address: Symbol, condition: JumpCondition}
}

#[derive(Debug)]
pub enum Code {
	Instruction(InstructionType),
	Comment(String)
}

#[derive(Debug)]
pub struct LineOfCode {
	pub line: usize,
	pub code: Result<Code, CompilerError>
}

#[derive(Debug)]
pub struct CompilerError {
	pub pos: lrpar::Span,
	pub description: CompilerErrorType
}

#[derive(Debug)]
pub enum CompilerErrorType {
	LiteralError(num::ParseIntError),
	LexerError(lrlex::DefaultLexeme),
	MiscError
}


impl From<num::ParseIntError> for CompilerErrorType {
	fn from(e: num::ParseIntError) -> Self {
		CompilerErrorType::LiteralError(e)
	}
}

impl From<lrlex::DefaultLexeme> for CompilerErrorType {
	fn from(e: lrlex::DefaultLexeme) -> Self {
		CompilerErrorType::LexerError(e)
	}
}

/*
impl fmt::Debug for LineOfCode {
	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
		write!(f, "Line = {}", self.line)
	}
}
*/
